@@ -1,6 +1,6 @@
 /* sane - Scanner Access Now Easy.
    Copyright (C) 1998 David F. Skoll
-   Heavily based on "hp.c" driver for HP Scanners, by 
+   Heavily based on "hp.c" driver for HP Scanners, by
    David Mosberger-Tang.
 
    This file is part of the SANE package.
@@ -16,9 +16,7 @@
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
    As a special exception, the authors of SANE give permission for
    additional uses of the libraries contained in this release of SANE.
@@ -44,8 +42,6 @@
    This file implements a SANE backend for the Polaroid Digital
    Microscope Camera. */
 
-/* $Id$ */
-
 #include "../include/sane/config.h"
 
 #include <limits.h>
@@ -113,7 +109,7 @@ static SANE_Word ValidASAs[] = { 3, 25, 50, 100 };
 // fd -- file descriptor
 // typecode -- data type code
 // qualifier -- data type qualifier
-// maxlen -- tranfer length
+// maxlen -- transfer length
 // buf -- buffer to store data in
 // len -- set to actual length of data
 //%RETURNS:
@@ -153,7 +149,7 @@ DMCRead(int fd, unsigned int typecode, unsigned int qualifier,
 // fd -- file descriptor
 // typecode -- data type code
 // qualifier -- data type qualifier
-// maxlen -- tranfer length
+// maxlen -- transfer length
 // buf -- buffer to store data in
 //%RETURNS:
 // A SANE status code
@@ -425,7 +421,7 @@ DMCInitOptions(DMC_Camera *c)
     c->opt[OPT_TL_X].constraint_type = SANE_CONSTRAINT_RANGE;
     c->opt[OPT_TL_X].constraint.range = &c->tl_x_range;
     c->val[OPT_TL_X].w = c->tl_x_range.min;
-    
+
     /* top-left y */
     c->opt[OPT_TL_Y].name = SANE_NAME_SCAN_TL_Y;
     c->opt[OPT_TL_Y].title = SANE_TITLE_SCAN_TL_Y;
@@ -435,7 +431,7 @@ DMCInitOptions(DMC_Camera *c)
     c->opt[OPT_TL_Y].constraint_type = SANE_CONSTRAINT_RANGE;
     c->opt[OPT_TL_Y].constraint.range = &c->tl_y_range;
     c->val[OPT_TL_Y].w = c->tl_y_range.min;
-    
+
     /* bottom-right x */
     c->opt[OPT_BR_X].name = SANE_NAME_SCAN_BR_X;
     c->opt[OPT_BR_X].title = SANE_TITLE_SCAN_BR_X;
@@ -445,7 +441,7 @@ DMCInitOptions(DMC_Camera *c)
     c->opt[OPT_BR_X].constraint_type = SANE_CONSTRAINT_RANGE;
     c->opt[OPT_BR_X].constraint.range = &c->br_x_range;
     c->val[OPT_BR_X].w = c->br_x_range.min;
-    
+
     /* bottom-right y */
     c->opt[OPT_BR_Y].name = SANE_NAME_SCAN_BR_Y;
     c->opt[OPT_BR_Y].title = SANE_TITLE_SCAN_BR_Y;
@@ -485,7 +481,7 @@ DMCInitOptions(DMC_Camera *c)
     c->opt[OPT_SHUTTER_SPEED].constraint_type = SANE_CONSTRAINT_RANGE;
     c->opt[OPT_SHUTTER_SPEED].constraint.range = &c->hw->shutterSpeedRange;
     c->val[OPT_SHUTTER_SPEED].w = c->hw->shutterSpeed;
-    
+
     c->opt[OPT_WHITE_BALANCE].name = "whitebalance";
     c->opt[OPT_WHITE_BALANCE].title = "White Balance";
     c->opt[OPT_WHITE_BALANCE].desc = "Selects white balance";
@@ -512,59 +508,65 @@ DMCInitOptions(DMC_Camera *c)
 static SANE_Status
 DMCSetMode(DMC_Camera *c, int mode)
 {
-    switch(mode) {
+  switch (mode)
+    {
     case IMAGE_MFI:
-	c->tl_x_range.min = 0;
-	c->tl_x_range.max = c->tl_x_range.max;
-	c->tl_y_range.min = 0;
-	c->tl_y_range.max = c->tl_y_range.max;
-	c->br_x_range.min = 800;
-	c->br_x_range.max = c->br_x_range.max;
-	c->br_y_range.min = 599;
-	c->br_y_range.max = c->br_y_range.max;
-	break;
+      c->tl_x_range.min = 0;
+      c->tl_x_range.max = 800;
+      c->tl_y_range.min = 0;
+      c->tl_y_range.max = 599;
+      c->br_x_range.min = c->tl_x_range.min;
+      c->br_x_range.max = c->tl_x_range.max;
+      c->br_y_range.min = c->tl_y_range.min;
+      c->br_y_range.max = c->tl_y_range.max;
+      break;
+
     case IMAGE_VIEWFINDER:
-	c->tl_x_range.min = 0;
-	c->tl_x_range.max = c->tl_x_range.max;
-	c->tl_y_range.min = 0;
-	c->tl_y_range.max = c->tl_y_range.max;
-	c->br_x_range.min = 269;
-	c->br_x_range.max = c->br_x_range.max;
-	c->br_y_range.min = 200;
-	c->br_y_range.max = c->br_y_range.max;
-	break;
+      c->tl_x_range.min = 0;
+      c->tl_x_range.max = 269;
+      c->tl_y_range.min = 0;
+      c->tl_y_range.max = 200;
+      c->br_x_range.min = c->tl_x_range.min;
+      c->br_x_range.max = c->tl_x_range.max;
+      c->br_y_range.min = c->tl_y_range.min;
+      c->br_y_range.max = c->tl_y_range.max;
+      break;
+
     case IMAGE_RAW:
-	c->tl_x_range.min = 0;
-	c->tl_x_range.max = c->tl_x_range.max;
-	c->tl_y_range.min = 0;
-	c->tl_y_range.max = c->tl_y_range.max;
-	c->br_x_range.min = 1598;
-	c->br_x_range.max = c->br_x_range.max;
-	c->br_y_range.min = 599;
-	c->br_y_range.max = c->br_y_range.max;
-	break;
+      c->tl_x_range.min = 0;
+      c->tl_x_range.max = 1598;
+      c->tl_y_range.min = 0;
+      c->tl_y_range.max = 599;
+      c->br_x_range.min = c->tl_x_range.min;
+      c->br_x_range.max = c->tl_x_range.max;
+      c->br_y_range.min = c->tl_y_range.min;
+      c->br_y_range.max = c->tl_y_range.max;
+      break;
+
     case IMAGE_THUMB:
-	c->tl_x_range.min = 0;
-	c->tl_x_range.max = c->tl_x_range.max;
-	c->tl_y_range.min = 0;
-	c->tl_y_range.max = c->tl_y_range.max;
-	c->br_x_range.min = 79;
-	c->br_x_range.max = c->br_x_range.max;
-	c->br_y_range.min = 59;
-	c->br_y_range.max = c->br_y_range.max;
-	break;
+      c->tl_x_range.min = 0;
+      c->tl_x_range.max = 79;
+      c->tl_y_range.min = 0;
+      c->tl_y_range.max = 59;
+      c->br_x_range.min = c->tl_x_range.min;
+      c->br_x_range.max = c->tl_x_range.max;
+      c->br_y_range.min = c->tl_y_range.min;
+      c->br_y_range.max = c->tl_y_range.max;
+      break;
+
     case IMAGE_SUPER_RES:
-	c->tl_x_range.min = 0;
-	c->tl_x_range.max = c->tl_x_range.max;
-	c->tl_y_range.min = 0;
-	c->tl_y_range.max = c->tl_y_range.max;
-	c->br_x_range.min = 1598;
-	c->br_x_range.max = c->br_x_range.max;
-	c->br_y_range.min = 1199;
-	c->br_y_range.max = c->br_y_range.max;
-	break;
+      c->tl_x_range.min = 0;
+      c->tl_x_range.max = 1598;
+      c->tl_y_range.min = 0;
+      c->tl_y_range.max = 1199;
+      c->br_x_range.min = c->tl_x_range.min;
+      c->br_x_range.max = c->tl_x_range.max;
+      c->br_y_range.min = c->tl_y_range.min;
+      c->br_y_range.max = c->tl_y_range.max;
+      break;
+
     default:
-	return SANE_STATUS_INVAL;
+      return SANE_STATUS_INVAL;
     }
     c->imageMode = mode;
     c->val[OPT_TL_X].w = c->tl_x_range.min;
@@ -618,7 +620,7 @@ DMCSetASA(int fd, unsigned int asa)
 
     if (i > ASA_100+1) return SANE_STATUS_INVAL;
 
-    status = DMCRead(fd, 0x87, 0x4, exposureCalculationResults, 
+    status = DMCRead(fd, 0x87, 0x4, exposureCalculationResults,
 		     sizeof(exposureCalculationResults), &len);
     if (status != SANE_STATUS_GOOD) return status;
     if (len < sizeof(exposureCalculationResults)) return SANE_STATUS_IO_ERROR;
@@ -647,7 +649,7 @@ DMCSetWhiteBalance(int fd, int mode)
     size_t len;
 
     DBG(3, "DMCSetWhiteBalance: %d\n", mode);
-    status = DMCRead(fd, 0x82, 0x0, userInterfaceSettings, 
+    status = DMCRead(fd, 0x82, 0x0, userInterfaceSettings,
 		     sizeof(userInterfaceSettings), &len);
     if (status != SANE_STATUS_GOOD) return status;
     if (len < sizeof(userInterfaceSettings)) return SANE_STATUS_IO_ERROR;
@@ -749,7 +751,7 @@ DMCReadTwoSuperResolutionLines(DMC_Camera *c, SANE_Byte *buf, int lastLine)
 	greenCoeff = ADVANCE_COEFF(greenCoeff);
 	blueCoeff = ADVANCE_COEFF(blueCoeff);
     }
-    
+
     /* Do the next super-resolution line and interpolate vertically */
     if (lastLine) {
 	memcpy(buf+BYTES_PER_RAW_LINE*3, buf, BYTES_PER_RAW_LINE*3);
@@ -868,7 +870,7 @@ sane_exit(void)
 	free(dev);
 	dev = next;
     }
-  
+
     if (devlist)
       free (devlist);
 }
@@ -1103,7 +1105,7 @@ sane_control_option(SANE_Handle handle, SANE_Int option,
 	if (c->val[OPT_SHUTTER_SPEED].w != * (SANE_Int *) val) {
 	    if (info) *info |= SANE_INFO_INEXACT;
 	}
-			
+
 	return SANE_STATUS_GOOD;
 
     default:
@@ -1328,7 +1330,7 @@ sane_read(SANE_Handle handle, SANE_Byte *buf, SANE_Int max_len, SANE_Int *len)
 	*len = max_len;
 	return SANE_STATUS_GOOD;
     }
-	
+
     if (c->imageMode == IMAGE_MFI || c->imageMode == IMAGE_RAW) {
 	/* We have to read complete rows... */
 	max_len = (max_len / c->params.bytes_per_line) * c->params.bytes_per_line;
