@@ -17,7 +17,7 @@
 #define USB_TIMEOUT (6 * 1000)
 #define USB_SHORT_TIMEOUT (1 * 800)
 
-size_t eds_send(epsonds_scanner *s, void *buf, size_t length, SANE_Status *status);
+size_t eds_send(epsonds_scanner *s, void *buf, size_t length, SANE_Status *status, size_t reply_len);
 size_t eds_recv(epsonds_scanner *s, void *buf, size_t length, SANE_Status *status);
 
 SANE_Status eds_txrx(epsonds_scanner *s, char *txbuf, size_t txlen,
@@ -30,4 +30,3 @@ SANE_Status eds_fsx(epsonds_scanner *s);
 SANE_Status eds_lock(epsonds_scanner *s);
 
 #endif
-
