@@ -18,6 +18,9 @@
 #define BACKEND_NAME epsonds
 #define DEBUG_NOT_STATIC
 
+#define mode_params epsonds_mode_params
+#define source_list epsonds_source_list
+
 #ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
 #endif
