@@ -18,9 +18,7 @@
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
    As a special exception, the authors of SANE give permission for
    additional uses of the libraries contained in this release of SANE.
@@ -624,7 +622,7 @@ sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
 /**
  * Open the backend, ie return the struct handle of a detected scanner
  * The struct returned is choosne if it matches the name given, which is
- * usefull when several scanners handled by the backend have been detected.
+ * useful when several scanners handled by the backend have been detected.
  * However, special case empty string "" and "lexmark" pick the first
  * available handle.
  */
@@ -649,7 +647,7 @@ sane_open (SANE_String_Const devicename, SANE_Handle * handle)
       return SANE_STATUS_INVAL;
     }
 
-  /* walk the linked list of scanner device until ther is a match
+  /* walk the linked list of scanner device until there is a match
    * with the device name */
   for (lexmark_device = first_lexmark_device; lexmark_device;
        lexmark_device = lexmark_device->next)
