@@ -1,6 +1,6 @@
 /* SANE - Scanner Access Now Easy.
 
-   Copyright (C) 2011-2019 Rolf Bensch <rolf at bensch hyphen online dot de>
+   Copyright (C) 2011-2020 Rolf Bensch <rolf at bensch hyphen online dot de>
    Copyright (C) 2007-2008 Nicolas Martin, <nicols-guest at alioth dot debian dot org>
    Copyright (C) 2006-2007 Wittawat Yamwong <wittawat@web.de>
 
@@ -17,9 +17,7 @@
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
    As a special exception, the authors of SANE give permission for
    additional uses of the libraries contained in this release of SANE.
@@ -815,7 +813,7 @@ static const pixma_scan_ops_t pixma_mp730_ops = {
               0x04a9, pid,       /* vid pid */	      \
               1,                 /* iface */		      \
               &pixma_mp730_ops,  /* ops */            \
-              0,                 /* min_xdpi not used in this subdriver */ \
+              0, 0,              /* min_xdpi & min_xdpi_16 not used in this subdriver */ \
               dpi, dpi,          /* xdpi, ydpi */	    \
               0, 0,              /* adftpu_min_dpi & adftpu_max_dpi not used in this subdriver */ \
               0, 0,              /* tpuir_min_dpi & tpuir_max_dpi not used in this subdriver */  \
