@@ -15,9 +15,7 @@
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
    As a special exception, the authors of SANE give permission for
    additional uses of the libraries contained in this release of SANE.
@@ -72,7 +70,6 @@
 #include "../include/sane/saneopts.h"
 
 #include <sys/ioctl.h>
-#include <asm/types.h>		/* XXX glibc */
 
 #define BACKEND_NAME v4l
 #include "../include/sane/sanei_backend.h"
@@ -495,13 +492,12 @@ sane_exit (void)
 }
 
 SANE_Status
-sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
+sane_get_devices (const SANE_Device *** device_list, SANE_Bool __sane_unused__ local_only)
 {
   V4L_Device *dev;
   int i;
 
   DBG (5, "sane_get_devices\n");
-  local_only = SANE_TRUE;	/* Avoid compile warning */
 
   if (devlist)
     free (devlist);
@@ -654,7 +650,7 @@ sane_open (SANE_String_Const devname, SANE_Handle * handle)
   DBG (5, "sane_open: x=%d, y=%d, width=%d, height=%d\n",
        s->window.x, s->window.y, s->window.width, s->window.height);
 
-  /* already done in sane_start 
+  /* already done in sane_start
      if (-1 == v4l1_ioctl (v4lfd, VIDIOCGMBUF, &mbuf))
      DBG (1, "sane_open: can't ioctl VIDIOCGMBUF (no Fbuffer?)\n");
    */
@@ -733,7 +729,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
        action == SANE_ACTION_GET_VALUE ? "get" :
        action == SANE_ACTION_SET_VALUE ? "set" :
        action == SANE_ACTION_SET_AUTO ? "auto set" :
-       "(unknow action with)", option,
+       "(unknown action with)", option,
        s->opt[option].name ? s->opt[option].name : s->opt[option].title);
 
   cap = s->opt[option].cap;
@@ -962,7 +958,7 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters * params)
 SANE_Status
 sane_start (SANE_Handle handle)
 {
-  int len, loop;
+  int len;
   V4L_Scanner *s;
   char data;
 
@@ -998,6 +994,7 @@ sane_start (SANE_Handle handle)
     }
   else
     {
+      int loop;
       s->is_mmap = SANE_TRUE;
       DBG (3,
 	   "sane_start: mmap frame, buffersize: %d bytes, buffers: %d, offset 0 %d\n",
@@ -1046,6 +1043,7 @@ sane_start (SANE_Handle handle)
   /* v4l1 actually returns BGR when we ask for RGB, so convert it */
   if (s->pict.palette == VIDEO_PALETTE_RGB24)
     {
+      uint32_t loop;
       DBG (3, "sane_start: converting from BGR to RGB\n");
       for (loop = 0; loop < (s->window.width * s->window.height * 3); loop += 3)
         {
@@ -1130,22 +1128,15 @@ sane_cancel (SANE_Handle handle)
 
 
 SANE_Status
-sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
+sane_set_io_mode (SANE_Handle __sane_unused__ handle, SANE_Bool non_blocking)
 {
-  /* Avoid compile warning */
-  handle = 0;
-
   if (non_blocking == SANE_FALSE)
     return SANE_STATUS_GOOD;
   return SANE_STATUS_UNSUPPORTED;
 }
 
 SANE_Status
-sane_get_select_fd (SANE_Handle handle, SANE_Int * fd)
+sane_get_select_fd (SANE_Handle __sane_unused__ handle, SANE_Int __sane_unused__ * fd)
 {
-  /* Avoid compile warning */
-  handle = 0;
-  fd = 0;
-
   return SANE_STATUS_UNSUPPORTED;
 }
